@@ -5,7 +5,7 @@ use warnings;
 
 # ABSTRACT: RapidApp-powered blog
 
-use RapidApp 1.3105;
+use RapidApp 1.3106;
 
 use Moose;
 extends 'RapidApp::Builder';
@@ -22,7 +22,7 @@ use YAML::XS 0.64 'LoadFile';
 use Rapi::Blog::Scaffold;
 use Rapi::Blog::Scaffold::Set;
 
-our $VERSION = '1.1001';
+our $VERSION = 1.1002;
 our $TITLE = "Rapi::Blog v" . $VERSION;
 
 has 'site_path',        is => 'ro', required => 1;
@@ -40,6 +40,9 @@ has 'underlay_scaffolds', is => 'ro', isa => ArrayRef[Str], default => sub {[]};
 has 'smtp_config', is => 'ro', isa => Maybe[HashRef], default => sub { undef };
 has 'override_email_recipient', is => 'ro', isa => Maybe[Str], default => sub { undef };
 
+has 'recaptcha_config', is => 'ro', isa => Maybe[HashRef[Str]], default => sub { undef };
+
+
 has '+base_appname', default => sub { 'Rapi::Blog::App' };
 has '+debug',        default => sub {1};
 
@@ -240,6 +243,30 @@ sub _get_underlay_scaffold_dirs {
 }
 
 
+sub _enforce_valid_recaptcha_config {
+  my $self = shift;
+  my $cfg = $self->recaptcha_config or return 1; # No config at all is valid
+  
+  my @valid_keys = qw/public_key private_key verify_url strict_mode/;
+  my %keys = map {$_=>1} @valid_keys;
+  for my $k (keys %$cfg) {
+    $keys{$k} or die join('',
+      "Unknown recaptcha_config param '$k' - ",
+      "only valid params are: ",join(', ',@valid_keys)
+    )
+  }
+  
+  die "Invalid recaptcha_config - both 'public_key' and 'private_key' params are required"
+    unless ($cfg->{public_key} && $cfg->{private_key});
+  
+  if(exists $cfg->{strict_mode}) {
+    my $v = $cfg->{strict_mode};
+    my $disp = defined $v ? "'$v'" : 'undef';
+    die "Bad value $disp for 'strict_mode' in recaptcha_config - must be either 1 (true) or 0 (false)\n"
+      unless ("$v" eq '0' || "$v" eq '1')
+  }
+}
+
 
 sub _build_version { $VERSION }
 sub _build_plugins { [qw/
@@ -252,6 +279,8 @@ sub _build_plugins { [qw/
 sub _build_base_config {
   my $self = shift;
   
+  $self->_enforce_valid_recaptcha_config;
+  
   my $tpl_dir = join('/',$self->share_dir,'templates');
   -d $tpl_dir or die join('',
     "template dir ($tpl_dir) not found; ", 
@@ -575,6 +604,43 @@ the default L<Email::Sender::Transport::Sendmail> options.
 If set, all e-mails generated by the system will be sent to the specified address instead of normal 
 recipients.
 
+=head2 recaptcha_config
+
+Optional HashRef config to enable Google reCAPTCHA v2 validation on supported forms. An account and API 
+key pair must be setup with Google first. This config supports the following params:
+
+=head3 public_key
+
+Required. The public, or "SITE KEY" provided by the Google reCAPTCHA settings, after being setup in the 
+Google reCAPTCHA system L<www.google.com/recaptcha/admin|http://www.google.com/recaptcha/admin>
+
+=head3 private_key
+
+Required. The private, or "SECRET KEY" provided by the Google reCAPTCHA settings, after being setup in the 
+Google reCAPTCHA system L<www.google.com/recaptcha/admin|http://www.google.com/recaptcha/admin>. Both the
+C<public_key> and the C<private_key> are provided as a pair and both are required.
+
+=head3 verify_url
+
+Optional URL to use when performing the actual reCAPCTHA validation with Google. Defaults to 
+C<https://www.google.com/recaptcha/api/siteverify> which should probably never need to be changed.
+
+=head3 strict_mode
+
+Optional mode (turned off by default) which can be enabled to tighten the enforcement reCAPTCHA, 
+requiring it in all locations which is is setup on the server side, regardless of whether or not the 
+client form is actually prompting the user with the appropriate reCAPTCHA "I am not a robot" checkbox
+dialog. In those cases of client-side forms not properly setup, they will never be able to submit 
+because they will always fail reCAPTCHA validation.
+
+When this mode is off (the default) reCAPTCHA validation is only performed when both the client and
+the server are properly setup. The downside of this is that it leaves open the scenario of a spammer
+direct posting to the server instead of using the actual form. Whether or not this mode should be 
+used should be based on need -- if spammers exploit this, turn it on. Otherwise, it is best to leave
+it off as it turning it on has the potential to make the site less resilient and reliable.
+
+This param accepts only 2 possible values: 1 (enabled) or 0 (disabled, which is the default).
+
 
 =head1 METHODS
 
